back_in_black = [
    {
        "youtube_link": "https://www.youtube.com/watch?v=9vWNauaZAgg",
        "raw_track_meta": {
            "album": {
                "album_type": "album",
                "artists": [
                    {
                        "external_urls": {
                            "spotify": "https://open.spotify.com/artist/711MCceyCBcFnzjGY4Q7Un"
                        },
                        "href": "https://api.spotify.com/v1/artists/711MCceyCBcFnzjGY4Q7Un",
                        "id": "711MCceyCBcFnzjGY4Q7Un",
                        "name": "AC/DC",
                        "type": "artist",
                        "uri": "spotify:artist:711MCceyCBcFnzjGY4Q7Un",
                    }
                ],
                "available_markets": [
                    "AD",
                    "AE",
                    "AL",
                    "AR",
                    "AT",
                    "AU",
                    "BA",
                    "BE",
                    "BG",
                    "BH",
                    "BO",
                    "BR",
                    "BY",
                    "CA",
                    "CH",
                    "CL",
                    "CO",
                    "CR",
                    "CY",
                    "CZ",
                    "DE",
                    "DK",
                    "DO",
                    "DZ",
                    "EC",
                    "EE",
                    "EG",
                    "ES",
                    "FI",
                    "FR",
                    "GB",
                    "GR",
                    "GT",
                    "HK",
                    "HN",
                    "HR",
                    "HU",
                    "ID",
                    "IE",
                    "IL",
                    "IN",
                    "IS",
                    "IT",
                    "JO",
                    "JP",
                    "KW",
                    "KZ",
                    "LB",
                    "LI",
                    "LT",
                    "LU",
                    "LV",
                    "MA",
                    "MC",
                    "MD",
                    "ME",
                    "MK",
                    "MT",
                    "MX",
                    "MY",
                    "NI",
                    "NL",
                    "NO",
                    "NZ",
                    "OM",
                    "PA",
                    "PE",
                    "PH",
                    "PL",
                    "PS",
                    "PT",
                    "PY",
                    "QA",
                    "RO",
                    "RS",
                    "RU",
                    "SA",
                    "SE",
                    "SG",
                    "SI",
                    "SK",
                    "SV",
                    "TH",
                    "TN",
                    "TR",
                    "TW",
                    "UA",
                    "US",
                    "UY",
                    "VN",
                    "XK",
                    "ZA",
                ],
                "external_urls": {
                    "spotify": "https://open.spotify.com/album/6mUdeDZCsExyJLMdAfDuwh"
                },
                "href": "https://api.spotify.com/v1/albums/6mUdeDZCsExyJLMdAfDuwh",
                "id": "6mUdeDZCsExyJLMdAfDuwh",
                "images": [
                    {
                        "height": 640,
                        "url": "https://i.scdn.co/image/ab67616d0000b2730b51f8d91f3a21e8426361ae",
                        "width": 640,
                    },
                    {
                        "height": 300,
                        "url": "https://i.scdn.co/image/ab67616d00001e020b51f8d91f3a21e8426361ae",
                        "width": 300,
                    },
                    {
                        "height": 64,
                        "url": "https://i.scdn.co/image/ab67616d000048510b51f8d91f3a21e8426361ae",
                        "width": 64,
                    },
                ],
                "name": "Back In Black",
                "release_date": "1980-07-25",
                "release_date_precision": "day",
                "total_tracks": 10,
                "type": "album",
                "uri": "spotify:album:6mUdeDZCsExyJLMdAfDuwh",
            },
            "artists": [
                {
                    "external_urls": {
                        "spotify": "https://open.spotify.com/artist/711MCceyCBcFnzjGY4Q7Un"
                    },
                    "href": "https://api.spotify.com/v1/artists/711MCceyCBcFnzjGY4Q7Un",
                    "id": "711MCceyCBcFnzjGY4Q7Un",
                    "name": "AC/DC",
                    "type": "artist",
                    "uri": "spotify:artist:711MCceyCBcFnzjGY4Q7Un",
                }
            ],
            "available_markets": [
                "AD",
                "AE",
                "AL",
                "AR",
                "AT",
                "AU",
                "BA",
                "BE",
                "BG",
                "BH",
                "BO",
                "BR",
                "BY",
                "CA",
                "CH",
                "CL",
                "CO",
                "CR",
                "CY",
                "CZ",
                "DE",
                "DK",
                "DO",
                "DZ",
                "EC",
                "EE",
                "EG",
                "ES",
                "FI",
                "FR",
                "GB",
                "GR",
                "GT",
                "HK",
                "HN",
                "HR",
                "HU",
                "ID",
                "IE",
                "IL",
                "IN",
                "IS",
                "IT",
                "JO",
                "JP",
                "KW",
                "KZ",
                "LB",
                "LI",
                "LT",
                "LU",
                "LV",
                "MA",
                "MC",
                "MD",
                "ME",
                "MK",
                "MT",
                "MX",
                "MY",
                "NI",
                "NL",
                "NO",
                "NZ",
                "OM",
                "PA",
                "PE",
                "PH",
                "PL",
                "PS",
                "PT",
                "PY",
                "QA",
                "RO",
                "RS",
                "RU",
                "SA",
                "SE",
                "SG",
                "SI",
                "SK",
                "SV",
                "TH",
                "TN",
                "TR",
                "TW",
                "UA",
                "US",
                "UY",
                "VN",
                "XK",
                "ZA",
            ],
            "disc_number": 1,
            "duration_ms": 255493,
            "explicit": False,
            "external_ids": {"isrc": "AUAP08000046"},
            "external_urls": {
                "spotify": "https://open.spotify.com/track/08mG3Y1vljYA6bvDt4Wqkj"
            },
            "href": "https://api.spotify.com/v1/tracks/08mG3Y1vljYA6bvDt4Wqkj",
            "id": "08mG3Y1vljYA6bvDt4Wqkj",
            "is_local": False,
            "name": "Back In Black",
            "popularity": 85,
            "preview_url": "https://p.scdn.co/mp3-preview/506bc9a0485990973449b0467791e06174371284?cid=4fe3fecfe5334023a1472516cc99d805",
            "track_number": 6,
            "type": "track",
            "uri": "spotify:track:08mG3Y1vljYA6bvDt4Wqkj",
        },
        "raw_album_meta": {
            "external_urls": {
                "spotify": "https://open.spotify.com/artist/711MCceyCBcFnzjGY4Q7Un"
            },
            "followers": {"href": None, "total": 16821071},
            "genres": ["album rock", "australian rock", "hard rock", "rock"],
            "href": "https://api.spotify.com/v1/artists/711MCceyCBcFnzjGY4Q7Un",
            "id": "711MCceyCBcFnzjGY4Q7Un",
            "images": [
                {
                    "height": 640,
                    "url": "https://i.scdn.co/image/52f9c4f83d8642ed80a0bad0317ab940e034de08",
                    "width": 640,
                },
                {
                    "height": 320,
                    "url": "https://i.scdn.co/image/64d25927322389160df14e1966849040954b29bb",
                    "width": 320,
                },
                {
                    "height": 160,
                    "url": "https://i.scdn.co/image/a7fa12b3b307ac3f172cf1760f385b44bc29e083",
                    "width": 160,
                },
            ],
            "name": "AC/DC",
            "popularity": 88,
            "type": "artist",
            "uri": "spotify:artist:711MCceyCBcFnzjGY4Q7Un",
        },
        "raw_artist_meta": {
            "external_urls": {
                "spotify": "https://open.spotify.com/artist/711MCceyCBcFnzjGY4Q7Un"
            },
            "followers": {"href": None, "total": 16821071},
            "genres": ["album rock", "australian rock", "hard rock", "rock"],
            "href": "https://api.spotify.com/v1/artists/711MCceyCBcFnzjGY4Q7Un",
            "id": "711MCceyCBcFnzjGY4Q7Un",
            "images": [
                {
                    "height": 640,
                    "url": "https://i.scdn.co/image/52f9c4f83d8642ed80a0bad0317ab940e034de08",
                    "width": 640,
                },
                {
                    "height": 320,
                    "url": "https://i.scdn.co/image/64d25927322389160df14e1966849040954b29bb",
                    "width": 320,
                },
                {
                    "height": 160,
                    "url": "https://i.scdn.co/image/a7fa12b3b307ac3f172cf1760f385b44bc29e083",
                    "width": 160,
                },
            ],
            "name": "AC/DC",
            "popularity": 88,
            "type": "artist",
            "uri": "spotify:artist:711MCceyCBcFnzjGY4Q7Un",
        },
        "lyrics": "[Verse 1]\nBack in black, I hit the sack\nI've been too long, I'm glad to be back\nYes, I'm let loose from the noose\nThat's kept me hanging about\nI'm just looking at the sky 'cause it's getting me high\nForget the hearse, 'cause I'll never die\nI got nine lives, cat's eyes\nAbusing every one of them and running wild\n\n[Chorus]\n'Cause I'm back, yes I'm back\nWell, I'm back, yes I'm back\nWell, I'm back, back\nWell, I'm back in black\nYes, I'm back in black\n\n[Verse 2]\nBack in the back of a Cadillac\nNumber one with a bullet, I'm a power pack\nYes, I'm in a bang with a gang\nThey've got to catch me if they want me to hang\n'Cause I'm back on the track and I'm beatin' the flack\nNobody's gonna get me on another rap\nSo, look at me now, I'm just making my play\nDon't try to push your luck, just get out of my way\n\n[Chorus]\n'Cause I'm back, yes I'm back\nWell, I'm back, yes I'm back\nWell, I'm back, back\nWell, I'm back in black\nYes, I'm back in black\n\n[Guitar Solo]\n\n[Chorus]\nWell, I'm back, yes I'm back\nWell, I'm back, yes I'm back\nWell, I'm back, back\nWell, I'm back in black\nYes, I'm back in black, yow\n\n[Bridge]\nAh, yeah\nOh, yeah\nTake my love\nYeah, yeah\nYeah\nAh, hey yeah\nOoh, yeah\n\n[Chorus]\nWell, I'm back (I'm back)\nBack, well I'm (I'm back)\nBack (I'm back)\nBack (I'm back)\nBack (I'm back)\nBack, back in black\nYes, I'm back in black\n\n[Outro]\nI've hit the sack\n\n[Guitar Solo]",
    }
]
